///////////////////////////////////////////////////////////////////////////////
//
// Texture Settings Dialog
// Copyright (C) 2006 Tobias Minch
//
// Part of rct3tool
// Copyright 2005 Jonathan Wilson
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Written by
//   Tobias Minich - belgabor@gmx.de
//
// Parts of this code are based on code written for rct3tool (Copyright 2005
// Jonathan Wilson).
//
///////////////////////////////////////////////////////////////////////////////

#include "wx_pch.h"

#include <wx/valtext.h>
#include <wx/valgen.h>

#include "wxtexturevalidator.h"
#include "auipicfiledlg.h"
#include "SCNFile.h"

#include "wxdlgTexture.h"
#include "ilhelper.h"

BEGIN_EVENT_TABLE(dlgTexture,wxDialog)
EVT_BUTTON(XRCID("m_btTextureInfo"), dlgTexture::OnTextureInfo)
EVT_BUTTON(XRCID("m_btAlphaInfo"), dlgTexture::OnAlphaInfo)
EVT_BUTTON(XRCID("m_btTextureOpen"), dlgTexture::OnTextureOpen)
EVT_BUTTON(XRCID("m_btAlphaOpen"), dlgTexture::OnAlphaOpen)
EVT_BUTTON(XRCID("m_btLoad"), dlgTexture::OnLoad)
END_EVENT_TABLE()

dlgTexture::dlgTexture(wxWindow *parent) {
    InitWidgetsFromXRC((wxWindow *)parent);
    m_textTextureName->SetValidator(wxTextValidator(wxFILTER_NONE, &m_ft.Name));
    m_textTextureFile->SetValidator(wxTextureValidator(_("Texture File"), false, &m_ft.Texture));
    m_textAlphaFile->SetValidator(wxTextureValidator(_("Alpha Texture File"), true, &m_ft.Alpha));
    m_choiceColors->SetValidator(wxGenericValidator(&m_recol));

    m_btOk->SetId(wxID_OK);
    m_btCancel->SetId(wxID_CANCEL);
}

void dlgTexture::SetFlexiTexture(cFlexiTexture val) {
    m_ft = val;
    m_recol = val.Recolorable;
    switch (m_recol) {
    case 3:
        m_recol = 2;
        break;
    case 7:
        m_recol = 3;
        break;
    }
}

cFlexiTexture dlgTexture::GetFlexiTexture() {
    cFlexiTexture ret = m_ft;
    ret.Recolorable = m_recol;
    switch (ret.Recolorable) {
    case 2:
        ret.Recolorable = 3;
        break;
    case 3:
        ret.Recolorable = 7;
        break;
    }
    return ret;
}

void dlgTexture::OnTextureInfo(wxCommandEvent& event) {
    if (m_textTextureFile->GetValue() == wxT("")) {
        ::wxMessageBox(_("Please select a texture file before requesting info on it ;-)"), _("Error"), wxICON_ERROR | wxOK, this);
    } else {
        showBitmapInfo(this, m_textTextureFile->GetValue().fn_str());
    }
}

void dlgTexture::OnAlphaInfo(wxCommandEvent& event) {
    if (m_textAlphaFile->GetValue() == wxT("")) {
        ::wxMessageBox(_("Please select an alpha texture file before requesting info on it ;-)"), _("Error"), wxICON_ERROR | wxOK, this);
    } else {
        showBitmapInfo(this, m_textAlphaFile->GetValue().fn_str());
    }
}

void dlgTexture::OnTextureOpen(wxCommandEvent& event) {
    wxFileName name = m_textTextureFile->GetValue();
    wxAUIPicFileDialog *dialog = new wxAUIPicFileDialog(
                                     NULL,
                                     _T("Select Texture File"),
                                     wxEmptyString,
                                     wxEmptyString,
                                     _T("Supported Image Files|*.png;*.jpg;*.bmp;*.gif;*.jpeg;*.tiff;*.tif|All Files (*.*)|*.*"),
                                     wxFD_FILE_MUST_EXIST|wxFD_CHANGE_DIR,
                                     wxDefaultPosition,
                                     wxSize(600,400)
                                 );
    if (name!=wxT("")) {
        dialog->SetFilename(name.GetFullName());
        dialog->SetDirectory(name.GetPath());
    }
    if (dialog->ShowModal() == wxID_OK) {
        m_textTextureFile->SetValue(dialog->GetPath());
        if (m_textTextureName->GetValue() == wxT("")) {
            name = dialog->GetPath();
            m_textTextureName->SetValue(name.GetName());
        }
    }
    delete dialog;
}

void dlgTexture::OnAlphaOpen(wxCommandEvent& event) {
    wxFileName name = m_textAlphaFile->GetValue();
    wxAUIPicFileDialog *dialog = new wxAUIPicFileDialog(
                                     NULL,
                                     _T("Select Alpha Texture File"),
                                     wxEmptyString,
                                     wxEmptyString,
                                     _T("Supported Image Files|*.png;*.jpg;*.bmp;*.gif;*.jpeg;*.tiff;*.tif|All Files (*.*)|*.*"),
                                     wxFD_FILE_MUST_EXIST|wxFD_CHANGE_DIR,
                                     wxDefaultPosition,
                                     wxSize(600,400)
                                 );
    if (name!=wxT("")) {
        dialog->SetFilename(name.GetFullName());
        dialog->SetDirectory(name.GetPath());
    }
    if (dialog->ShowModal() == wxID_OK) {
        m_textAlphaFile->SetValue(dialog->GetPath());
    }
    delete dialog;
}

void dlgTexture::OnLoad(wxCommandEvent& event) {
    wxFileDialog *dialog = new wxFileDialog(
                                     NULL,
                                     _T("Open Scenery File (Load texture settings)"),
                                     wxEmptyString,
                                     wxEmptyString,
                                     _T("Scenery Files (*.scn)|*.scn"),
                                     wxFD_FILE_MUST_EXIST|wxFD_CHANGE_DIR,
                                     wxDefaultPosition,
                                     wxSize(600,400)
                                 );
    if (dialog->ShowModal() == wxID_OK) {
        cSCNFile *texscn = new cSCNFile(dialog->GetPath().fn_str());
        if (texscn) {
            if (texscn->flexitextures.size()==0) {
                wxMessageBox(_("The selected scenery file contains no textures."), _("Warning"), wxOK | wxICON_WARNING, this);
                delete texscn;
                delete dialog;
                return;
            }
            wxArrayString choices;
            for (cFlexiTextureIterator ms = texscn->flexitextures.begin(); ms != texscn->flexitextures.end(); ms++) {
                choices.Add(ms->Name);
            }
            if (choices.size() > 0) {
                int choice = ::wxGetSingleChoiceIndex(dialog->GetPath(), _("Choose texture settings to import"), choices, this);
                if (choice >= 0) {
                    m_textTextureName->SetValue(texscn->flexitextures[choice].Name);
                    m_textTextureFile->SetValue(texscn->flexitextures[choice].Texture);
                    m_textAlphaFile->SetValue(texscn->flexitextures[choice].Alpha);
                    long Recolor = texscn->flexitextures[choice].Recolorable;
                    switch (Recolor) {
                    case 3:
                        Recolor = 2;
                        break;
                    case 7:
                        Recolor = 3;
                        break;
                    }
                    m_choiceColors->SetSelection(Recolor);
                }
            }
            delete texscn;
        }
    }
    delete dialog;
}

